from django.urls import path, include
from . import views

app_name = 'netbox_zwave'
urlpatterns = [
    # Networks
    path('networks/', views.ZwaveNetworkListView.as_view(), name='zwavenetwork_list'),
    path('networks/add/', views.ZwaveNetworkEditView.as_view(), name='zwavenetwork_add'),
    path('networks/<int:pk>/', views.ZwaveNetworkView.as_view(), name='zwavenetwork'),
    path('networks/<int:pk>/edit/', views.ZwaveNetworkEditView.as_view(), name='zwavenetwork_edit'),
    path('networks/<int:pk>/delete/', views.ZwaveNetworkDeleteView.as_view(), name='zwavenetwork_delete'),
    
    # Devices
    path('devices/', views.ZwaveDeviceListView.as_view(), name='zwavedevice_list'),
    path('devices/add/', views.ZwaveDeviceEditView.as_view(), name='zwavedevice_add'),
    path('devices/<int:pk>/', views.ZwaveDeviceView.as_view(), name='zwavedevice'),
    path('devices/<int:pk>/edit/', views.ZwaveDeviceEditView.as_view(), name='zwavedevice_edit'),
    path('devices/<int:pk>/delete/', views.ZwaveDeviceDeleteView.as_view(), name='zwavedevice_delete'),
    
    # Command Classes
    path('command-classes/', views.ZwaveCommandClassListView.as_view(), name='zwavecommandclass_list'),
    path('command-classes/add/', views.ZwaveCommandClassEditView.as_view(), name='zwavecommandclass_add'),
    path('command-classes/<int:pk>/', views.ZwaveCommandClassView.as_view(), name='zwavecommandclass'),
    path('command-classes/<int:pk>/edit/', views.ZwaveCommandClassEditView.as_view(), name='zwavecommandclass_edit'),
    path('command-classes/<int:pk>/delete/', views.ZwaveCommandClassDeleteView.as_view(), name='zwavecommandclass_delete'),
    
    # Associations
    path('associations/', views.ZwaveAssociationListView.as_view(), name='zwaveassociation_list'),
    path('associations/add/', views.ZwaveAssociationEditView.as_view(), name='zwaveassociation_add'),
    path('associations/<int:pk>/', views.ZwaveAssociationView.as_view(), name='zwaveassociation'),
    path('associations/<int:pk>/edit/', views.ZwaveAssociationEditView.as_view(), name='zwaveassociation_edit'),
    path('associations/<int:pk>/delete/', views.ZwaveAssociationDeleteView.as_view(), name='zwaveassociation_delete'),
]



from django import forms
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from utilities.forms.fields import DynamicModelChoiceField, DynamicModelMultipleChoiceField
from .models import ZwaveNetwork, ZwaveDevice, ZwaveCommandClass, ZwaveAssociation


class ZwaveNetworkForm(NetBoxModelForm):
    """Form for Z-Wave Network"""
    
    class Meta:
        model = ZwaveNetwork
        fields = [
            'name', 'network_id', 'home_id', 'status', 'description',
            'controller_node_id', 'security_enabled', 's2_enabled', 'tags'
        ]
        widgets = {
            'description': forms.Textarea(attrs={'rows': 3}),
        }


class ZwaveNetworkFilterForm(NetBoxModelFilterSetForm):
    """Filter form for Z-Wave Network"""
    model = ZwaveNetwork
    fields = ['id', 'name', 'network_id', 'home_id', 'status', 'security_enabled', 's2_enabled']


class ZwaveDeviceForm(NetBoxModelForm):
    """Form for Z-Wave Device"""
    network = DynamicModelChoiceField(
        queryset=ZwaveNetwork.objects.all(),
        required=True,
        help_text="Z-Wave network this device belongs to"
    )
    
    class Meta:
        model = ZwaveDevice
        fields = [
            'network', 'name', 'node_id', 'device_type', 'status',
            'manufacturer_id', 'product_type', 'product_id', 'firmware_version',
            'hardware_version', 'library_type', 'protocol_version', 'application_version',
            'is_secure', 'is_frequent_listener', 'is_beaming', 'is_routing',
            'max_baud_rate', 'last_seen', 'battery_level', 'signal_strength',
            'description', 'tags'
        ]
        widgets = {
            'description': forms.Textarea(attrs={'rows': 3}),
            'last_seen': forms.DateTimeInput(attrs={'type': 'datetime-local'}),
        }


class ZwaveDeviceFilterForm(NetBoxModelFilterSetForm):
    """Filter form for Z-Wave Device"""
    model = ZwaveDevice
    fields = [
        'id', 'network', 'name', 'node_id', 'device_type', 'status',
        'manufacturer_id', 'product_type', 'product_id', 'is_secure',
        'is_frequent_listener', 'is_beaming', 'is_routing'
    ]


class ZwaveCommandClassForm(NetBoxModelForm):
    """Form for Z-Wave Command Class"""
    device = DynamicModelChoiceField(
        queryset=ZwaveDevice.objects.all(),
        required=True,
        help_text="Device this command class belongs to"
    )
    
    class Meta:
        model = ZwaveCommandClass
        fields = [
            'device', 'command_class_id', 'name', 'version',
            'is_secure', 'is_controlled', 'is_supported', 'tags'
        ]


class ZwaveCommandClassFilterForm(NetBoxModelFilterSetForm):
    """Filter form for Z-Wave Command Class"""
    model = ZwaveCommandClass
    fields = [
        'id', 'device', 'command_class_id', 'name', 'version',
        'is_secure', 'is_controlled', 'is_supported'
    ]


class ZwaveAssociationForm(NetBoxModelForm):
    """Form for Z-Wave Association"""
    source_device = DynamicModelChoiceField(
        queryset=ZwaveDevice.objects.all(),
        required=True,
        help_text="Source device"
    )
    target_device = DynamicModelChoiceField(
        queryset=ZwaveDevice.objects.all(),
        required=True,
        help_text="Target device"
    )
    
    class Meta:
        model = ZwaveAssociation
        fields = [
            'source_device', 'target_device', 'group_id', 'description', 'tags'
        ]
        widgets = {
            'description': forms.Textarea(attrs={'rows': 3}),
        }


class ZwaveAssociationFilterForm(NetBoxModelFilterSetForm):
    """Filter form for Z-Wave Association"""
    model = ZwaveAssociation
    fields = ['id', 'source_device', 'target_device', 'group_id']



# Generated by Django 5.2.6

from django.db import migrations, models
import django.core.validators
import netbox.models
import utilities.choices


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='ZwaveNetwork',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict)),
                ('name', models.CharField(help_text='Name of the Z-Wave network', max_length=100)),
                ('network_id', models.PositiveIntegerField(help_text='Z-Wave Network ID (1-255)', unique=True, validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(255)])),
                ('home_id', models.CharField(help_text='Z-Wave Home ID (hexadecimal)', max_length=16, unique=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('inactive', 'Inactive'), ('maintenance', 'Maintenance')], default='active', help_text='Network status', max_length=20)),
                ('description', models.TextField(blank=True, help_text='Network description')),
                ('controller_node_id', models.PositiveIntegerField(blank=True, help_text='Controller node ID', null=True, validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(232)])),
                ('security_enabled', models.BooleanField(default=False, help_text='Whether security is enabled on this network')),
                ('s2_enabled', models.BooleanField(default=False, help_text='Whether S2 security is enabled')),
                ('tags', models.ManyToManyField(blank=True, related_name='+', to='extras.tag')),
            ],
            options={
                'verbose_name': 'Z-Wave Network',
                'verbose_name_plural': 'Z-Wave Networks',
                'ordering': ['name'],
            },
            bases=(netbox.models.NetBoxModel, models.Model),
        ),
        migrations.CreateModel(
            name='ZwaveDevice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict)),
                ('name', models.CharField(help_text='Device name', max_length=100)),
                ('node_id', models.PositiveIntegerField(help_text='Z-Wave Node ID (1-232)', validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(232)])),
                ('device_type', models.CharField(choices=[('controller', 'Controller'), ('router', 'Router'), ('end_device', 'End Device'), ('sleeping_end_device', 'Sleeping End Device')], default='end_device', help_text='Device type', max_length=20)),
                ('status', models.CharField(choices=[('online', 'Online'), ('offline', 'Offline'), ('unknown', 'Unknown'), ('failed', 'Failed')], default='unknown', help_text='Device status', max_length=20)),
                ('manufacturer_id', models.PositiveIntegerField(blank=True, help_text='Manufacturer ID', null=True)),
                ('product_type', models.PositiveIntegerField(blank=True, help_text='Product Type', null=True)),
                ('product_id', models.PositiveIntegerField(blank=True, help_text='Product ID', null=True)),
                ('firmware_version', models.CharField(blank=True, help_text='Firmware version', max_length=20)),
                ('hardware_version', models.CharField(blank=True, help_text='Hardware version', max_length=20)),
                ('library_type', models.CharField(blank=True, help_text='Library type', max_length=20)),
                ('protocol_version', models.CharField(blank=True, help_text='Protocol version', max_length=20)),
                ('application_version', models.CharField(blank=True, help_text='Application version', max_length=20)),
                ('is_secure', models.BooleanField(default=False, help_text='Whether device supports security')),
                ('is_frequent_listener', models.BooleanField(default=False, help_text='Whether device is a frequent listener')),
                ('is_beaming', models.BooleanField(default=False, help_text='Whether device supports beaming')),
                ('is_routing', models.BooleanField(default=False, help_text='Whether device supports routing')),
                ('max_baud_rate', models.PositiveIntegerField(blank=True, help_text='Maximum baud rate', null=True)),
                ('last_seen', models.DateTimeField(blank=True, help_text='Last time device was seen', null=True)),
                ('battery_level', models.PositiveIntegerField(blank=True, help_text='Battery level (0-100)', null=True, validators=[django.core.validators.MaxValueValidator(100)])),
                ('signal_strength', models.IntegerField(blank=True, help_text='Signal strength (dBm)', null=True)),
                ('description', models.TextField(blank=True, help_text='Device description')),
                ('network', models.ForeignKey(help_text='Z-Wave network this device belongs to', on_delete=models.CASCADE, related_name='devices', to='netbox_zwave.zwavenetwork')),
                ('tags', models.ManyToManyField(blank=True, related_name='+', to='extras.tag')),
            ],
            options={
                'verbose_name': 'Z-Wave Device',
                'verbose_name_plural': 'Z-Wave Devices',
                'ordering': ['network', 'node_id'],
            },
            bases=(netbox.models.NetBoxModel, models.Model),
        ),
        migrations.CreateModel(
            name='ZwaveCommandClass',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict)),
                ('command_class_id', models.PositiveIntegerField(help_text='Command Class ID', validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(255)])),
                ('name', models.CharField(help_text='Command class name', max_length=100)),
                ('version', models.PositiveIntegerField(default=1, help_text='Command class version')),
                ('is_secure', models.BooleanField(default=False, help_text='Whether command class is secure')),
                ('is_controlled', models.BooleanField(default=False, help_text='Whether command class is controlled')),
                ('is_supported', models.BooleanField(default=True, help_text='Whether command class is supported')),
                ('device', models.ForeignKey(help_text='Device this command class belongs to', on_delete=models.CASCADE, related_name='command_classes', to='netbox_zwave.zwavedevice')),
                ('tags', models.ManyToManyField(blank=True, related_name='+', to='extras.tag')),
            ],
            options={
                'verbose_name': 'Z-Wave Command Class',
                'verbose_name_plural': 'Z-Wave Command Classes',
                'ordering': ['device', 'command_class_id'],
            },
            bases=(netbox.models.NetBoxModel, models.Model),
        ),
        migrations.CreateModel(
            name='ZwaveAssociation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict)),
                ('group_id', models.PositiveIntegerField(default=1, help_text='Association group ID')),
                ('description', models.TextField(blank=True, help_text='Association description')),
                ('source_device', models.ForeignKey(help_text='Source device', on_delete=models.CASCADE, related_name='source_associations', to='netbox_zwave.zwavedevice')),
                ('target_device', models.ForeignKey(help_text='Target device', on_delete=models.CASCADE, related_name='target_associations', to='netbox_zwave.zwavedevice')),
                ('tags', models.ManyToManyField(blank=True, related_name='+', to='extras.tag')),
            ],
            options={
                'verbose_name': 'Z-Wave Association',
                'verbose_name_plural': 'Z-Wave Associations',
                'ordering': ['source_device', 'group_id'],
            },
            bases=(netbox.models.NetBoxModel, models.Model),
        ),
    ]



from django.shortcuts import get_object_or_404
from django.contrib import messages
from django.http import HttpResponseRedirect
from django.urls import reverse
from netbox.views import generic
from netbox.views.generic import ObjectListView, ObjectView, ObjectEditView, ObjectDeleteView
from . import filtersets, forms, models, tables


class ZwaveNetworkListView(ObjectListView):
    """List view for Z-Wave Networks"""
    queryset = models.ZwaveNetwork.objects.all()
    table = tables.ZwaveNetworkTable
    filterset = filtersets.ZwaveNetworkFilterSet
    filterset_form = forms.ZwaveNetworkFilterForm
    template_name = 'netbox_zwave/zwavenetwork_list.html'


class ZwaveNetworkView(ObjectView):
    """Detail view for Z-Wave Network"""
    queryset = models.ZwaveNetwork.objects.all()
    template_name = 'netbox_zwave/zwavenetwork.html'

    def get_extra_context(self, request, instance):
        return {
            'devices_table': tables.ZwaveDeviceTable(instance.devices.all()),
        }


class ZwaveNetworkEditView(ObjectEditView):
    """Create/Edit view for Z-Wave Network"""
    queryset = models.ZwaveNetwork.objects.all()
    form = forms.ZwaveNetworkForm
    template_name = 'netbox_zwave/zwavenetwork_edit.html'


class ZwaveNetworkDeleteView(ObjectDeleteView):
    """Delete view for Z-Wave Network"""
    queryset = models.ZwaveNetwork.objects.all()
    template_name = 'netbox_zwave/zwavenetwork_delete.html'


class ZwaveDeviceListView(ObjectListView):
    """List view for Z-Wave Devices"""
    queryset = models.ZwaveDevice.objects.select_related('network').all()
    table = tables.ZwaveDeviceTable
    filterset = filtersets.ZwaveDeviceFilterSet
    filterset_form = forms.ZwaveDeviceFilterForm
    template_name = 'netbox_zwave/zwavedevice_list.html'


class ZwaveDeviceView(ObjectView):
    """Detail view for Z-Wave Device"""
    queryset = models.ZwaveDevice.objects.select_related('network').all()
    template_name = 'netbox_zwave/zwavedevice.html'

    def get_extra_context(self, request, instance):
        return {
            'command_classes_table': tables.ZwaveCommandClassTable(instance.command_classes.all()),
            'source_associations_table': tables.ZwaveAssociationTable(instance.source_associations.all()),
            'target_associations_table': tables.ZwaveAssociationTable(instance.target_associations.all()),
        }


class ZwaveDeviceEditView(ObjectEditView):
    """Create/Edit view for Z-Wave Device"""
    queryset = models.ZwaveDevice.objects.all()
    form = forms.ZwaveDeviceForm
    template_name = 'netbox_zwave/zwavedevice_edit.html'


class ZwaveDeviceDeleteView(ObjectDeleteView):
    """Delete view for Z-Wave Device"""
    queryset = models.ZwaveDevice.objects.all()
    template_name = 'netbox_zwave/zwavedevice_delete.html'


class ZwaveCommandClassListView(ObjectListView):
    """List view for Z-Wave Command Classes"""
    queryset = models.ZwaveCommandClass.objects.select_related('device').all()
    table = tables.ZwaveCommandClassTable
    filterset = filtersets.ZwaveCommandClassFilterSet
    filterset_form = forms.ZwaveCommandClassFilterForm
    template_name = 'netbox_zwave/zwavecommandclass_list.html'


class ZwaveCommandClassView(ObjectView):
    """Detail view for Z-Wave Command Class"""
    queryset = models.ZwaveCommandClass.objects.select_related('device').all()
    template_name = 'netbox_zwave/zwavecommandclass.html'


class ZwaveCommandClassEditView(ObjectEditView):
    """Create/Edit view for Z-Wave Command Class"""
    queryset = models.ZwaveCommandClass.objects.all()
    form = forms.ZwaveCommandClassForm
    template_name = 'netbox_zwave/zwavecommandclass_edit.html'


class ZwaveCommandClassDeleteView(ObjectDeleteView):
    """Delete view for Z-Wave Command Class"""
    queryset = models.ZwaveCommandClass.objects.all()
    template_name = 'netbox_zwave/zwavecommandclass_delete.html'


class ZwaveAssociationListView(ObjectListView):
    """List view for Z-Wave Associations"""
    queryset = models.ZwaveAssociation.objects.select_related('source_device', 'target_device').all()
    table = tables.ZwaveAssociationTable
    filterset = filtersets.ZwaveAssociationFilterSet
    filterset_form = forms.ZwaveAssociationFilterForm
    template_name = 'netbox_zwave/zwaveassociation_list.html'


class ZwaveAssociationView(ObjectView):
    """Detail view for Z-Wave Association"""
    queryset = models.ZwaveAssociation.objects.select_related('source_device', 'target_device').all()
    template_name = 'netbox_zwave/zwaveassociation.html'


class ZwaveAssociationEditView(ObjectEditView):
    """Create/Edit view for Z-Wave Association"""
    queryset = models.ZwaveAssociation.objects.all()
    form = forms.ZwaveAssociationForm
    template_name = 'netbox_zwave/zwaveassociation_edit.html'


class ZwaveAssociationDeleteView(ObjectDeleteView):
    """Delete view for Z-Wave Association"""
    queryset = models.ZwaveAssociation.objects.all()
    template_name = 'netbox_zwave/zwaveassociation_delete.html'



from django.urls import path, include
from rest_framework.routers import DefaultRouter
from . import views

router = DefaultRouter()
router.register('networks', views.ZwaveNetworkViewSet)
router.register('devices', views.ZwaveDeviceViewSet)
router.register('command-classes', views.ZwaveCommandClassViewSet)
router.register('associations', views.ZwaveAssociationViewSet)

app_name = 'netbox_zwave-api'
urlpatterns = [
    path('', include(router.urls)),
]



from rest_framework import viewsets
from netbox.api.viewsets import NetBoxModelViewSet
from ..models import ZwaveNetwork, ZwaveDevice, ZwaveCommandClass, ZwaveAssociation
from .serializers import (
    ZwaveNetworkSerializer,
    ZwaveDeviceSerializer,
    ZwaveCommandClassSerializer,
    ZwaveAssociationSerializer
)


class ZwaveNetworkViewSet(NetBoxModelViewSet):
    """ViewSet for Z-Wave Network model"""
    queryset = ZwaveNetwork.objects.all()
    serializer_class = ZwaveNetworkSerializer
    filterset_fields = ['name', 'network_id', 'home_id', 'status', 'security_enabled', 's2_enabled']
    search_fields = ['name', 'description', 'home_id']


class ZwaveDeviceViewSet(NetBoxModelViewSet):
    """ViewSet for Z-Wave Device model"""
    queryset = ZwaveDevice.objects.select_related('network').all()
    serializer_class = ZwaveDeviceSerializer
    filterset_fields = [
        'network', 'name', 'node_id', 'device_type', 'status',
        'manufacturer_id', 'product_type', 'product_id', 'is_secure',
        'is_frequent_listener', 'is_beaming', 'is_routing'
    ]
    search_fields = ['name', 'description', 'manufacturer_id', 'product_type', 'product_id']


class ZwaveCommandClassViewSet(NetBoxModelViewSet):
    """ViewSet for Z-Wave Command Class model"""
    queryset = ZwaveCommandClass.objects.select_related('device').all()
    serializer_class = ZwaveCommandClassSerializer
    filterset_fields = [
        'device', 'command_class_id', 'name', 'version',
        'is_secure', 'is_controlled', 'is_supported'
    ]
    search_fields = ['name']


class ZwaveAssociationViewSet(NetBoxModelViewSet):
    """ViewSet for Z-Wave Association model"""
    queryset = ZwaveAssociation.objects.select_related('source_device', 'target_device').all()
    serializer_class = ZwaveAssociationSerializer
    filterset_fields = [
        'source_device', 'target_device', 'group_id'
    ]
    search_fields = ['description']



from netbox.plugins import PluginConfig


class NetBoxZwaveConfig(PluginConfig):
    name = 'netbox_zwave'
    verbose_name = 'NetBox Z-Wave'
    description = 'A NetBox plugin for tracking Z-Wave devices and networks'
    version = '0.1.0'
    author = 'Richard Dawson'
    author_email = 'dawsora@gmail.com'
    base_url = 'zwave'
    required_settings = []
    default_settings = {
        'enable_auto_discovery': False,
        'default_network_id': 1,
        'device_timeout': 30,
    }
    django_apps = []
    min_version = '4.0'
    max_version = '4.99'


config = NetBoxZwaveConfig



from django.db import models
from django.core.validators import MinValueValidator, MaxValueValidator
from django.urls import reverse
from netbox.models import NetBoxModel
from utilities.choices import ChoiceSet


class ZwaveNetworkStatusChoices(ChoiceSet):
    """Choices for Z-Wave network status"""
    ACTIVE = 'active', 'Active'
    INACTIVE = 'inactive', 'Inactive'
    MAINTENANCE = 'maintenance', 'Maintenance'
    
    CHOICES = (
        ACTIVE,
        INACTIVE,
        MAINTENANCE,
    )


class ZwaveDeviceStatusChoices(ChoiceSet):
    """Choices for Z-Wave device status"""
    ONLINE = 'online', 'Online'
    OFFLINE = 'offline', 'Offline'
    UNKNOWN = 'unknown', 'Unknown'
    FAILED = 'failed', 'Failed'
    
    CHOICES = (
        ONLINE,
        OFFLINE,
        UNKNOWN,
        FAILED,
    )


class ZwaveDeviceTypeChoices(ChoiceSet):
    """Choices for Z-Wave device types"""
    CONTROLLER = 'controller', 'Controller'
    ROUTER = 'router', 'Router'
    END_DEVICE = 'end_device', 'End Device'
    SLEEPING_END_DEVICE = 'sleeping_end_device', 'Sleeping End Device'
    
    CHOICES = (
        CONTROLLER,
        ROUTER,
        END_DEVICE,
        SLEEPING_END_DEVICE,
    )


class ZwaveNetwork(NetBoxModel):
    """Z-Wave Network model"""
    name = models.CharField(
        max_length=100,
        help_text="Name of the Z-Wave network"
    )
    network_id = models.PositiveIntegerField(
        unique=True,
        validators=[MinValueValidator(1), MaxValueValidator(255)],
        help_text="Z-Wave Network ID (1-255)"
    )
    home_id = models.CharField(
        max_length=16,
        unique=True,
        help_text="Z-Wave Home ID (hexadecimal)"
    )
    status = models.CharField(
        max_length=20,
        choices=ZwaveNetworkStatusChoices,
        default=ZwaveNetworkStatusChoices.ACTIVE,
        help_text="Network status"
    )
    description = models.TextField(
        blank=True,
        help_text="Network description"
    )
    controller_node_id = models.PositiveIntegerField(
        null=True,
        blank=True,
        validators=[MinValueValidator(1), MaxValueValidator(232)],
        help_text="Controller node ID"
    )
    security_enabled = models.BooleanField(
        default=False,
        help_text="Whether security is enabled on this network"
    )
    s2_enabled = models.BooleanField(
        default=False,
        help_text="Whether S2 security is enabled"
    )

    class Meta:
        ordering = ['name']
        verbose_name = 'Z-Wave Network'
        verbose_name_plural = 'Z-Wave Networks'

    def __str__(self):
        return f"{self.name} (ID: {self.network_id})"

    def get_absolute_url(self):
        return reverse('plugins:netbox_zwave:zwavenetwork', kwargs={'pk': self.pk})


class ZwaveDevice(NetBoxModel):
    """Z-Wave Device model"""
    network = models.ForeignKey(
        to=ZwaveNetwork,
        on_delete=models.CASCADE,
        related_name='devices',
        help_text="Z-Wave network this device belongs to"
    )
    name = models.CharField(
        max_length=100,
        help_text="Device name"
    )
    node_id = models.PositiveIntegerField(
        validators=[MinValueValidator(1), MaxValueValidator(232)],
        help_text="Z-Wave Node ID (1-232)"
    )
    device_type = models.CharField(
        max_length=30,
        choices=ZwaveDeviceTypeChoices,
        default=ZwaveDeviceTypeChoices.END_DEVICE,
        help_text="Device type"
    )
    status = models.CharField(
        max_length=20,
        choices=ZwaveDeviceStatusChoices,
        default=ZwaveDeviceStatusChoices.UNKNOWN,
        help_text="Device status"
    )
    manufacturer_id = models.CharField(
        max_length=8,
        blank=True,
        help_text="Manufacturer ID (hexadecimal)"
    )
    product_type = models.CharField(
        max_length=8,
        blank=True,
        help_text="Product Type (hexadecimal)"
    )
    product_id = models.CharField(
        max_length=8,
        blank=True,
        help_text="Product ID (hexadecimal)"
    )
    firmware_version = models.CharField(
        max_length=20,
        blank=True,
        help_text="Firmware version"
    )
    hardware_version = models.CharField(
        max_length=20,
        blank=True,
        help_text="Hardware version"
    )
    library_type = models.CharField(
        max_length=20,
        blank=True,
        help_text="Z-Wave library type"
    )
    protocol_version = models.CharField(
        max_length=10,
        blank=True,
        help_text="Z-Wave protocol version"
    )
    application_version = models.CharField(
        max_length=10,
        blank=True,
        help_text="Application version"
    )
    is_secure = models.BooleanField(
        default=False,
        help_text="Whether device supports security"
    )
    is_frequent_listener = models.BooleanField(
        default=False,
        help_text="Whether device is a frequent listener"
    )
    is_beaming = models.BooleanField(
        default=False,
        help_text="Whether device supports beaming"
    )
    is_routing = models.BooleanField(
        default=False,
        help_text="Whether device supports routing"
    )
    max_baud_rate = models.PositiveIntegerField(
        null=True,
        blank=True,
        help_text="Maximum baud rate"
    )
    last_seen = models.DateTimeField(
        null=True,
        blank=True,
        help_text="Last time device was seen"
    )
    battery_level = models.PositiveIntegerField(
        null=True,
        blank=True,
        validators=[MinValueValidator(0), MaxValueValidator(100)],
        help_text="Battery level percentage (0-100)"
    )
    signal_strength = models.IntegerField(
        null=True,
        blank=True,
        help_text="Signal strength in dBm"
    )
    description = models.TextField(
        blank=True,
        help_text="Device description"
    )

    class Meta:
        ordering = ['network', 'node_id']
        unique_together = ['network', 'node_id']
        verbose_name = 'Z-Wave Device'
        verbose_name_plural = 'Z-Wave Devices'

    def __str__(self):
        return f"{self.name} (Node {self.node_id})"

    def get_absolute_url(self):
        return reverse('plugins:netbox_zwave:zwavedevice', kwargs={'pk': self.pk})


class ZwaveCommandClass(NetBoxModel):
    """Z-Wave Command Class model"""
    device = models.ForeignKey(
        to=ZwaveDevice,
        on_delete=models.CASCADE,
        related_name='command_classes',
        help_text="Device this command class belongs to"
    )
    command_class_id = models.PositiveIntegerField(
        validators=[MinValueValidator(0), MaxValueValidator(255)],
        help_text="Command Class ID (0-255)"
    )
    name = models.CharField(
        max_length=100,
        help_text="Command class name"
    )
    version = models.PositiveIntegerField(
        default=1,
        help_text="Command class version"
    )
    is_secure = models.BooleanField(
        default=False,
        help_text="Whether command class supports security"
    )
    is_controlled = models.BooleanField(
        default=False,
        help_text="Whether device can control this command class"
    )
    is_supported = models.BooleanField(
        default=True,
        help_text="Whether device supports this command class"
    )

    class Meta:
        ordering = ['device', 'command_class_id']
        unique_together = ['device', 'command_class_id']
        verbose_name = 'Z-Wave Command Class'
        verbose_name_plural = 'Z-Wave Command Classes'

    def __str__(self):
        return f"{self.name} (v{self.version})"

    def get_absolute_url(self):
        return reverse('plugins:netbox_zwave:zwavecommandclass', kwargs={'pk': self.pk})


class ZwaveAssociation(NetBoxModel):
    """Z-Wave Association model"""
    source_device = models.ForeignKey(
        to=ZwaveDevice,
        on_delete=models.CASCADE,
        related_name='source_associations',
        help_text="Source device"
    )
    target_device = models.ForeignKey(
        to=ZwaveDevice,
        on_delete=models.CASCADE,
        related_name='target_associations',
        help_text="Target device"
    )
    group_id = models.PositiveIntegerField(
        validators=[MinValueValidator(1), MaxValueValidator(255)],
        help_text="Association group ID"
    )
    description = models.TextField(
        blank=True,
        help_text="Association description"
    )

    class Meta:
        ordering = ['source_device', 'group_id', 'target_device']
        unique_together = ['source_device', 'target_device', 'group_id']
        verbose_name = 'Z-Wave Association'
        verbose_name_plural = 'Z-Wave Associations'

    def __str__(self):
        return f"{self.source_device} -> {self.target_device} (Group {self.group_id})"

    def get_absolute_url(self):
        return reverse('plugins:netbox_zwave:zwaveassociation', kwargs={'pk': self.pk})



from django.db.models import Q
from netbox.filtersets import NetBoxModelFilterSet
from .models import ZwaveNetwork, ZwaveDevice, ZwaveCommandClass, ZwaveAssociation


class ZwaveNetworkFilterSet(NetBoxModelFilterSet):
    """Filter set for Z-Wave Network"""
    
    class Meta:
        model = ZwaveNetwork
        fields = ['id', 'name', 'network_id', 'home_id', 'status', 'security_enabled', 's2_enabled']

    def search(self, queryset, name, value):
        """Search across multiple fields"""
        if not value.strip():
            return queryset
        return queryset.filter(
            Q(name__icontains=value) |
            Q(description__icontains=value) |
            Q(home_id__icontains=value)
        )


class ZwaveDeviceFilterSet(NetBoxModelFilterSet):
    """Filter set for Z-Wave Device"""
    
    class Meta:
        model = ZwaveDevice
        fields = [
            'id', 'network', 'name', 'node_id', 'device_type', 'status',
            'manufacturer_id', 'product_type', 'product_id', 'is_secure',
            'is_frequent_listener', 'is_beaming', 'is_routing'
        ]

    def search(self, queryset, name, value):
        """Search across multiple fields"""
        if not value.strip():
            return queryset
        return queryset.filter(
            Q(name__icontains=value) |
            Q(description__icontains=value) |
            Q(manufacturer_id__icontains=value) |
            Q(product_type__icontains=value) |
            Q(product_id__icontains=value)
        )


class ZwaveCommandClassFilterSet(NetBoxModelFilterSet):
    """Filter set for Z-Wave Command Class"""
    
    class Meta:
        model = ZwaveCommandClass
        fields = [
            'id', 'device', 'command_class_id', 'name', 'version',
            'is_secure', 'is_controlled', 'is_supported'
        ]

    def search(self, queryset, name, value):
        """Search across multiple fields"""
        if not value.strip():
            return queryset
        return queryset.filter(
            Q(name__icontains=value)
        )


class ZwaveAssociationFilterSet(NetBoxModelFilterSet):
    """Filter set for Z-Wave Association"""
    
    class Meta:
        model = ZwaveAssociation
        fields = ['id', 'source_device', 'target_device', 'group_id']

    def search(self, queryset, name, value):
        """Search across multiple fields"""
        if not value.strip():
            return queryset
        return queryset.filter(
            Q(description__icontains=value)
        )



from rest_framework import serializers
from netbox.api.serializers import NetBoxModelSerializer
from ..models import ZwaveNetwork, ZwaveDevice, ZwaveCommandClass, ZwaveAssociation


class ZwaveNetworkSerializer(NetBoxModelSerializer):
    """Serializer for Z-Wave Network model"""
    device_count = serializers.IntegerField(read_only=True)
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_zwave-api:zwavenetwork-detail'
    )

    class Meta:
        model = ZwaveNetwork
        fields = [
            'id', 'url', 'display', 'name', 'network_id', 'home_id', 'status',
            'description', 'controller_node_id', 'security_enabled', 's2_enabled',
            'device_count', 'created', 'last_updated', 'tags', 'custom_fields'
        ]


class ZwaveDeviceSerializer(NetBoxModelSerializer):
    """Serializer for Z-Wave Device model"""
    network_name = serializers.CharField(source='network.name', read_only=True)
    network_id = serializers.IntegerField(source='network.network_id', read_only=True)
    command_class_count = serializers.IntegerField(read_only=True)
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_zwave-api:zwavedevice-detail'
    )

    class Meta:
        model = ZwaveDevice
        fields = [
            'id', 'url', 'display', 'network', 'network_name', 'network_id',
            'name', 'node_id', 'device_type', 'status', 'manufacturer_id',
            'product_type', 'product_id', 'firmware_version', 'hardware_version',
            'library_type', 'protocol_version', 'application_version',
            'is_secure', 'is_frequent_listener', 'is_beaming', 'is_routing',
            'max_baud_rate', 'last_seen', 'battery_level', 'signal_strength',
            'description', 'command_class_count', 'created', 'last_updated',
            'tags', 'custom_fields'
        ]


class ZwaveCommandClassSerializer(NetBoxModelSerializer):
    """Serializer for Z-Wave Command Class model"""
    device_name = serializers.CharField(source='device.name', read_only=True)
    device_node_id = serializers.IntegerField(source='device.node_id', read_only=True)
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_zwave-api:zwavecommandclass-detail'
    )

    class Meta:
        model = ZwaveCommandClass
        fields = [
            'id', 'url', 'display', 'device', 'device_name', 'device_node_id',
            'command_class_id', 'name', 'version', 'is_secure', 'is_controlled',
            'is_supported', 'created', 'last_updated', 'tags', 'custom_fields'
        ]


class ZwaveAssociationSerializer(NetBoxModelSerializer):
    """Serializer for Z-Wave Association model"""
    source_device_name = serializers.CharField(source='source_device.name', read_only=True)
    source_device_node_id = serializers.IntegerField(source='source_device.node_id', read_only=True)
    target_device_name = serializers.CharField(source='target_device.name', read_only=True)
    target_device_node_id = serializers.IntegerField(source='target_device.node_id', read_only=True)
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_zwave-api:zwaveassociation-detail'
    )

    class Meta:
        model = ZwaveAssociation
        fields = [
            'id', 'url', 'display', 'source_device', 'source_device_name',
            'source_device_node_id', 'target_device', 'target_device_name',
            'target_device_node_id', 'group_id', 'description', 'created',
            'last_updated', 'tags', 'custom_fields'
        ]



import django_tables2 as tables
from django_tables2.utils import Accessor
from netbox.tables import NetBoxTable, columns
from .models import ZwaveNetwork, ZwaveDevice, ZwaveCommandClass, ZwaveAssociation


class ZwaveNetworkTable(NetBoxTable):
    """Table for Z-Wave Networks"""
    name = tables.Column(
        linkify=True,
        verbose_name='Name'
    )
    network_id = tables.Column(
        verbose_name='Network ID'
    )
    home_id = tables.Column(
        verbose_name='Home ID'
    )
    status = columns.ChoiceFieldColumn(
        verbose_name='Status'
    )
    device_count = tables.Column(
        verbose_name='Devices',
        accessor=Accessor('devices.count'),
        orderable=False
    )
    security_enabled = columns.BooleanColumn(
        verbose_name='Security'
    )
    s2_enabled = columns.BooleanColumn(
        verbose_name='S2'
    )
    tags = columns.TagColumn(
        url_name='plugins:netbox_zwave:zwavenetwork_list'
    )

    class Meta(NetBoxTable.Meta):
        model = ZwaveNetwork
        fields = (
            'pk', 'id', 'name', 'network_id', 'home_id', 'status',
            'device_count', 'security_enabled', 's2_enabled', 'created',
            'last_updated', 'tags'
        )
        default_columns = (
            'name', 'network_id', 'home_id', 'status', 'device_count',
            'security_enabled', 's2_enabled'
        )


class ZwaveDeviceTable(NetBoxTable):
    """Table for Z-Wave Devices"""
    name = tables.Column(
        linkify=True,
        verbose_name='Name'
    )
    network = tables.Column(
        linkify=True,
        verbose_name='Network'
    )
    node_id = tables.Column(
        verbose_name='Node ID'
    )
    device_type = columns.ChoiceFieldColumn(
        verbose_name='Type'
    )
    status = columns.ChoiceFieldColumn(
        verbose_name='Status'
    )
    manufacturer_id = tables.Column(
        verbose_name='Manufacturer ID'
    )
    product_type = tables.Column(
        verbose_name='Product Type'
    )
    product_id = tables.Column(
        verbose_name='Product ID'
    )
    firmware_version = tables.Column(
        verbose_name='Firmware'
    )
    is_secure = columns.BooleanColumn(
        verbose_name='Secure'
    )
    is_routing = columns.BooleanColumn(
        verbose_name='Routing'
    )
    battery_level = tables.Column(
        verbose_name='Battery %'
    )
    signal_strength = tables.Column(
        verbose_name='Signal (dBm)'
    )
    last_seen = columns.DateTimeColumn(
        verbose_name='Last Seen'
    )
    command_class_count = tables.Column(
        verbose_name='Command Classes',
        accessor=Accessor('command_classes.count'),
        orderable=False
    )
    tags = columns.TagColumn(
        url_name='plugins:netbox_zwave:zwavedevice_list'
    )

    class Meta(NetBoxTable.Meta):
        model = ZwaveDevice
        fields = (
            'pk', 'id', 'name', 'network', 'node_id', 'device_type', 'status',
            'manufacturer_id', 'product_type', 'product_id', 'firmware_version',
            'is_secure', 'is_routing', 'battery_level', 'signal_strength',
            'last_seen', 'command_class_count', 'created', 'last_updated', 'tags'
        )
        default_columns = (
            'name', 'network', 'node_id', 'device_type', 'status',
            'manufacturer_id', 'firmware_version', 'is_secure', 'battery_level'
        )


class ZwaveCommandClassTable(NetBoxTable):
    """Table for Z-Wave Command Classes"""
    device = tables.Column(
        linkify=True,
        verbose_name='Device'
    )
    command_class_id = tables.Column(
        verbose_name='Command Class ID'
    )
    name = tables.Column(
        verbose_name='Name'
    )
    version = tables.Column(
        verbose_name='Version'
    )
    is_secure = columns.BooleanColumn(
        verbose_name='Secure'
    )
    is_controlled = columns.BooleanColumn(
        verbose_name='Controlled'
    )
    is_supported = columns.BooleanColumn(
        verbose_name='Supported'
    )
    tags = columns.TagColumn(
        url_name='plugins:netbox_zwave:zwavecommandclass_list'
    )

    class Meta(NetBoxTable.Meta):
        model = ZwaveCommandClass
        fields = (
            'pk', 'id', 'device', 'command_class_id', 'name', 'version',
            'is_secure', 'is_controlled', 'is_supported', 'created',
            'last_updated', 'tags'
        )
        default_columns = (
            'device', 'command_class_id', 'name', 'version',
            'is_secure', 'is_controlled', 'is_supported'
        )


class ZwaveAssociationTable(NetBoxTable):
    """Table for Z-Wave Associations"""
    source_device = tables.Column(
        linkify=True,
        verbose_name='Source Device'
    )
    target_device = tables.Column(
        linkify=True,
        verbose_name='Target Device'
    )
    group_id = tables.Column(
        verbose_name='Group ID'
    )
    tags = columns.TagColumn(
        url_name='plugins:netbox_zwave:zwaveassociation_list'
    )

    class Meta(NetBoxTable.Meta):
        model = ZwaveAssociation
        fields = (
            'pk', 'id', 'source_device', 'target_device', 'group_id',
            'description', 'created', 'last_updated', 'tags'
        )
        default_columns = (
            'source_device', 'target_device', 'group_id', 'description'
        )



from netbox.plugins import PluginMenu, PluginMenuItem, PluginMenuButton


menu_items = (
    PluginMenuItem(
        link='plugins:netbox_zwave:zwavenetwork_list',
        link_text='Networks',
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_zwave:zwavenetwork_add',
                title='Add Network',
                icon_class='mdi mdi-plus-thick',
                color='green',
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_zwave:zwavedevice_list',
        link_text='Devices',
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_zwave:zwavedevice_add',
                title='Add Device',
                icon_class='mdi mdi-plus-thick',
                color='green',
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_zwave:zwavecommandclass_list',
        link_text='Command Classes',
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_zwave:zwavecommandclass_add',
                title='Add Command Class',
                icon_class='mdi mdi-plus-thick',
                color='green',
            ),
        ),
    ),
    PluginMenuItem(
        link='plugins:netbox_zwave:zwaveassociation_list',
        link_text='Associations',
        buttons=(
            PluginMenuButton(
                link='plugins:netbox_zwave:zwaveassociation_add',
                title='Add Association',
                icon_class='mdi mdi-plus-thick',
                color='green',
            ),
        ),
    ),
)
